"""
Fonctions intégrées à QGIS, reprises et appellées pour le fonctionnement du module
"""
import os

from qgis import *
from qgis.PyQt import *
from qgis.core import *
from PyQt5.QtGui import *
from PyQt5.QtCore import *
from processing.tools import dataobjects
import processing
from .Utilitaires import *
import grassprovider

# This loads your .ui file so that PyQt can populate your plugin with the elements from Qt Designer
path=os.path.dirname(os.path.abspath(__file__))
path=path.replace("Fonctions",'Siligites_dialog_base.ui')
FORM_CLASS, _ = uic.loadUiType(path)

class Fonctions_QGIS(QtWidgets.QDialog, FORM_CLASS) :
    def __init__(self, parent=None):
        """Constructor."""
        super(Fonctions_QGIS, self).__init__(parent)
        # Set up the user interface from Designer through FORM_CLASS.
        # After self.setupUi() you can access any designer object by doing
        # self.<objectname>, and you can use autoconnect slots - see
        # http://qt-project.org/doc/qt-4.8/designer-using-a-ui-file.html
        # #widgets-and-dialogs-with-auto-connect
        self.setupUi(self)


    def Selectbylocation_simple(self,Entree,Intersect,Methode=0):
        """
        Entrées : self
                  Entree (VectorLayer): Couche qui sera modifiée
                  Intersect (VectorLayer): Couche servant pour la comparaison
                  Méthode (int): 0 = Nouvelle sélection, 1 = Ajouter à la sélection, 2 = Sélection dans la sélection, 3 = Retirer de la sélection
        Fonction : Sélection par localisation de la couche entière
        """
        config = processing.ProcessingConfig
        config.setSettingValue('FILTER_INVALID_GEOMETRIES', 1)
        Params = {'INPUT':Entree,
                  'PREDICATE':[0],
                  'INTERSECT':Intersect,
                  'METHOD':Methode}
        processing.run("native:selectbylocation", Params)
        config.setSettingValue('FILTER_INVALID_GEOMETRIES', 2)

    
    def Selectbylocation(self,Entree,Intersect,Selected=True,Methode=1) :
        """
        Entrées : self
                  Entree (VectorLayer): Couche qui sera modifiée
                  Intersect (VectorLayer): Couche servant pour la comparaison
                  Selected (bool): N'utiliser que les attributs sélectionnés
                  Méthode (int): 0 = Nouvelle sélection, 1 = Ajouter à la sélection, 2 = Sélection dans la sélection, 3 = Retirer de la sélection
        Fonction : Sélection par localisation grâce aux entités sélectionnées d'une couche
        """
        config = processing.ProcessingConfig
        config.setSettingValue('FILTER_INVALID_GEOMETRIES', 1)
        Params = {'INPUT':Entree,
                  'PREDICATE':[0],
                  'INTERSECT':QgsProcessingFeatureSourceDefinition(Intersect.source(), 
                                                                 selectedFeaturesOnly=Selected, 
                                                                 featureLimit=-1, 
                                                                 geometryCheck=QgsFeatureRequest.GeometryNoCheck),
                  'METHOD':Methode}
        processing.run("native:selectbylocation", Params)
        config.setSettingValue('FILTER_INVALID_GEOMETRIES', 2)

    
    def Selectbyattribute(self,Entree,Champ,Valeur,Methode=0):
        """
        Entrées : self
                  Entree (VectorLayer): Couche qui sera modifiée
                  Champ (str): Champ de référence pour la sélection
                  Valeur (int ou float): Valeur de référence pour la sélection
                  Méthode (int): 0 = Nouvelle sélection, 1 = Ajouter à la sélection, 2 = Retirer de la sélection, 3 = Sélection dans la sélection
        Fonction : Sélection par attribut dans une couche
        """
        config = processing.ProcessingConfig
        config.setSettingValue('FILTER_INVALID_GEOMETRIES', 1)
        Params={'INPUT': Entree,
                'FIELD': Champ,
                'OPERATOR':0,
                'VALUE': Valeur,
                'METHOD':Methode}
        processing.run("qgis:selectbyattribute", Params)
        config.setSettingValue('FILTER_INVALID_GEOMETRIES', 2)

    
    def Buffer(self,Entree,Distance,Selected=True,Dissolve=True):
        """
        Entrées : self
                  Entree (VectorLayer): Couche servant de base pour le tampon/érosion
                  Distance (int): Distance du buffer (en m)
                  Selected (bool): N'utiliser que les attributs sélectionnés
                  Dissolve (bool): Fusionner les buffer si contact
        Fonction : Création d'un tampon autour d'un point
        Sortie : Tampon/érosion créé(e) (VectorLayer)
        """
        config = processing.ProcessingConfig
        config.setSettingValue('FILTER_INVALID_GEOMETRIES', 1)
        Params= {'INPUT':QgsProcessingFeatureSourceDefinition(Entree.source(), 
                                                              selectedFeaturesOnly=Selected, 
                                                              featureLimit=-1, 
                                                              geometryCheck=QgsFeatureRequest.GeometryAbortOnInvalid),
                 'DISTANCE':Distance,
                 'SEGMENTS':200,
                 'END_CAP_STYLE':0,
                 'JOIN_STYLE':0,
                 'MITER_LIMIT':2,
                 'DISSOLVE':Dissolve,
                 'SEPARATE_DISJOINT':False,
                 'OUTPUT':'TEMPORARY_OUTPUT'}
        
        resulttemp=processing.run("native:buffer", Params)
        config.setSettingValue('FILTER_INVALID_GEOMETRIES', 2)
        return resulttemp['OUTPUT']
    

    def Difference(self,Min,Max):
        """
        Entrées : self
                  Min (int ou float): Buffer avec la distance minimale
                  Max (int ou float): Buffer avec la distance maximale
        Fonction : Créer un anneau entre deux distance min et max
        Sortie : Surface issue de la différence (VectorLayer)
        """
        config = processing.ProcessingConfig
        config.setSettingValue('FILTER_INVALID_GEOMETRIES', 1)
        Params={'INPUT':Max,
                'OVERLAY':Min,
                'OUTPUT':'TEMPORARY_OUTPUT',
                'GRID_SIZE':None}
        resulttemp=processing.run("native:difference",Params)
        config.setSettingValue('FILTER_INVALID_GEOMETRIES', 2)
        return resulttemp['OUTPUT']
    

    def Agregation(self,Entree):
        """
        Entrées : self
                  Entree (VectorLayer): Couche qui sera aggrégée
        Fonction : Aggrégation des entités d'une couche en fonction d'un champ 
                   (Les paramètres choisis ici sont en lien avec les objectifs du module)
        Sortie : Couche contenant les entités aggrégées (VectorLayer)
        """
        config = processing.ProcessingConfig
        config.setSettingValue('FILTER_INVALID_GEOMETRIES', 1)
        Params={'INPUT':Entree,
                'GROUP_BY':'"ETAGE_FINAL"',
                'AGGREGATES':[{'aggregate': 'concatenate_unique','delimiter': ',','input': '"notation"','length': 4000,'name': 'notation','precision': 0,'sub_type': 0,'type': 10,'type_name': 'text'},
                              {'aggregate': 'concatenate_unique','delimiter': ',','input': '"descr"','length': 4000,'name': 'descr','precision': 0,'sub_type': 0,'type': 10,'type_name': 'text'},
                              {'aggregate': 'concatenate_unique','delimiter': ',','input': '"ere"','length': 4000,'name': 'ere','precision': 0,'sub_type': 0,'type': 10,'type_name': 'text'},
                              {'aggregate': 'concatenate_unique','delimiter': ',','input': '"serie"','length': 4000,'name': 'serie','precision': 0,'sub_type': 0,'type': 10,'type_name': 'text'},
                              {'aggregate': 'concatenate_unique','delimiter': ',','input': '"etage"','length': 4000,'name': 'etage','precision': 0,'sub_type': 0,'type': 10,'type_name': 'text'},
                              {'aggregate': 'concatenate_unique','delimiter': ',','input': '"type_gite"','length': 4000,'name': 'type_gite','precision': 0,'sub_type': 0,'type': 10,'type_name': 'text'},
                              {'aggregate': 'concatenate_unique','delimiter': ',','input': '"nom_num_carte"','length': 4000,'name': 'nom_num_carte','precision': 0,'sub_type': 0,'type': 10,'type_name': 'text'},
                              {'aggregate': 'concatenate_unique','delimiter': ',','input': '"etage_final"','length': 4000,'name': 'etage_final','precision': 0,'sub_type': 0,'type': 10,'type_name': 'text'}],
                'OUTPUT':'TEMPORARY_OUTPUT'}
        resulttemp=processing.run("native:aggregate",Params)
        config.setSettingValue('FILTER_INVALID_GEOMETRIES', 2)
        return resulttemp['OUTPUT']
    

    def Agregation_simple(self,Entree):
        """
        Entrées : self
                  Entree (VectorLayer): Couche dont il faut fusionner les entités
        Fonction : Fusion des entités d'une VectorLayer, sans prise en compte des champs qu'elle contient dans sa table attributaire
        Sortie : Couche où les entités ont été rassemblées en une seule (VectorLayer)
        """
        config = processing.ProcessingConfig
        config.setSettingValue('FILTER_INVALID_GEOMETRIES', 1)
        Params={'INPUT':Entree,
                'GROUP_BY':'NULL',
                'AGGREGATES':[],
                'OUTPUT':'TEMPORARY_OUTPUT'}
        resulttemp=processing.run("native:aggregate",Params)
        config.setSettingValue('FILTER_INVALID_GEOMETRIES', 2)
        return resulttemp['OUTPUT']
    

    def Centroide(self,Entree):
        """
        Entrées : self
                  Entree (VectorLayer): Couche de polygones servant de base pour les centroïdes
        Fonction : Création de centroïdes au sein de polygones
        Sortie : Couche contenant les centroïdes (VectorLayer)
        """
        config = processing.ProcessingConfig
        config.setSettingValue('FILTER_INVALID_GEOMETRIES', 1)
        Params={'INPUT': Entree,
                'ALL_PARTS':False,
                'OUTPUT':'TEMPORARY_OUTPUT'}
        resulttemp=processing.run("native:centroids",Params)
        config.setSettingValue('FILTER_INVALID_GEOMETRIES', 2)
        return resulttemp['OUTPUT']
    
    
    def Extraction_Sommets(self,Entree):
        """
        Entrées : self
                  Entree (VectorLayer): Couche de polygones dont les sommets seront extraits
        Fonction : Extraction des sommets de polygones
        Sortie : Couche contenant les sommets extraits (VectorLayer)
        """
        Params={'INPUT':Entree,
                'OUTPUT':'TEMPORARY_OUTPUT'}
        resulttemp=processing.run("native:extractvertices",Params)
        return resulttemp['OUTPUT']
    

    def Reprojeter(self,Entree,SCR):
        """
        Entrées : self
                  Entree (VectorLayer): Couche vecteur à reprojeter
                  SCR (int ou str): SCR de reprojection
        Fonction : Outil de reprojection de couches vecteurs
        Sortie : Couche reprojetée (VectorLayer)
        """
        Params={'INPUT':Entree,
                'TARGET_CRS':QgsCoordinateReferenceSystem('EPSG:{}'.format(SCR)),
                'OUTPUT':'TEMPORARY_OUTPUT'}
        resulttemps=processing.run("native:reprojectlayer",Params)
        return resulttemps['OUTPUT']


    def Centralite(self,Entree):
        """
        Entrées : self
                  Entree (VectorLayer): Réseau à analyser
        Fonction : Analyse de centralité d'un réseau grace à l'outil v.net.centrality
        Sortie : Couche de points contenant les informations de centralité (VectorLayer)
        """
        try :
            print(grassprovider.Grass7Utils.Grass7Utils.installedVersion())
        except :
            self.v_GRASS="grass"
        else:
            self.v_GRASS="grass7"

        path=os.path.dirname(os.path.abspath(__file__))
        path=path.replace('Fonctions','Couches//T//Centralite.gpkg')

        Params={'input':Entree,
                'degree':'degree',
                'closeness':'closeness',
                'betweenness':'betweenness',
                'eigenvector':'eigenvector',
                'iterations':1000,
                'error':0.1,
                'cats':'',
                'where':'',
                'arc_column':'',
                'arc_backward_column':'',
                'node_column':'',
                '-a':True,
                '-g':False,
                'output':path,
                'GRASS_REGION_PARAMETER':None,
                'GRASS_SNAP_TOLERANCE_PARAMETER':-1,
                'GRASS_MIN_AREA_PARAMETER':0.0001,
                'GRASS_OUTPUT_TYPE_PARAMETER':0,
                'GRASS_VECTOR_DSCO':'',
                'GRASS_VECTOR_LCO':'',
                'GRASS_VECTOR_EXPORT_NOCAT':False}
        resulttemps=processing.run(self.v_GRASS+":v.net.centrality",Params)

        # La couche de sortie ne pouvant être utilisée que si elle est enregisrée, celle-ci est créée dans un des dossiers
        path2=resulttemps['output']
        output_layer = QgsVectorLayer(path2,"Centralite",'ogr')

        # Création d'une couche temporaire pour la remplacer et faciliter son enregistrement par la suite
        SCR=output_layer.crs().authid().replace("EPSG:","")
        Couche_finale=Utilitaires.Creation_Couche(self,"Point",SCR,"Noeuds_reseau_centralite",output_layer)

        # QgsProject.instance().addMapLayer(output_layer)   
        output_layer.selectAll()

        Layer_data = Couche_finale.dataProvider()
        Layer_data.addFeatures(output_layer.selectedFeatures())
        # QgsProject.instance().removeMapLayer(output_layer)

        return Couche_finale
    

    def Noeuds(self,Entree):
        """
        Entrées : self
                  Entree (VectorLayer): Reseau à analyser
        Fonction : Récupération des noeuds du réseau grace à l'outil v.net
        Sortie : Couche de points contenant les noeuds du réseau (VectorLayer)
        """
        try :
            print(grassprovider.Grass7Utils.Grass7Utils.installedVersion())
        except :
            self.v_GRASS="grass"
        else:
            self.v_GRASS="grass7"

        path=os.path.dirname(os.path.abspath(__file__))
        path=path.replace('Fonctions','Couches//T//Noeuds.gpkg')

        Params={'input':Entree,
                'points':None,
                'file':'',
                'operation':0,
                'threshold':50,
                'arc_type':[0,1],
                '-s':False,
                '-c':False,
                'output':path,
                'GRASS_REGION_PARAMETER':None,
                'GRASS_SNAP_TOLERANCE_PARAMETER':-1,
                'GRASS_MIN_AREA_PARAMETER':0.0001,
                'GRASS_OUTPUT_TYPE_PARAMETER':0,
                'GRASS_VECTOR_DSCO':'',
                'GRASS_VECTOR_LCO':'',
                'GRASS_VECTOR_EXPORT_NOCAT':False}
        resulttemps=processing.run(self.v_GRASS+":v.net",Params)

        # La couche de sortie ne pouvant être utilisée que si elle est enregisrée, celle-ci est créée dans un des dossiers
        path2=resulttemps['output']
        output_layer = QgsVectorLayer(path2,"Noeuds",'ogr')

        # Création d'une couche temporaire pour la remplacer et faciliter son enregistrement par la suite
        SCR=output_layer.crs().authid().replace("EPSG:","")
        Couche_finale=Utilitaires.Creation_Couche(self,"Point",SCR,"Noeuds",output_layer)

        # QgsProject.instance().addMapLayer(output_layer)   
        output_layer.selectAll()

        Layer_data = Couche_finale.dataProvider()
        Layer_data.addFeatures(output_layer.selectedFeatures())
        # QgsProject.instance().removeLayer(output_layer)

        return Couche_finale


    def Chemin(self,Reseau,Depart,Arrivee):
        """
        Entrées : self
                  Reseau (VectorLayer): Réseau à parcourir
                  Depart (str): Point de départ ('X,Y [EPSG:xxxx]')
                  Arrivee (str): Point d'arrivee ('X,Y [EPSG:xxxx]')
        Fonction : Recherche u chemin le plus court dans un réseau entre deux points
        Sortie : Chemin le plus court identifié (VectorLayer)
        """
        Params={'INPUT':Reseau,
                'STRATEGY':1,
                'DIRECTION_FIELD':'',
                'VALUE_FORWARD':'',
                'VALUE_BACKWARD':'',
                'VALUE_BOTH':'',
                'DEFAULT_DIRECTION':2,
                'SPEED_FIELD':'',
                'DEFAULT_SPEED':1.5,
                'TOLERANCE':0,
                'START_POINT':Depart,
                'END_POINT':Arrivee,
                'OUTPUT':'TEMPORARY_OUTPUT'}
        resulttemps=processing.run("native:shortestpathpointtopoint",Params)

        return resulttemps['OUTPUT']
    

    def Select_Egal(self,Entree,Intersect,Methode):
        """
        Entrées : self
                  Entree (VectorLayer): Couche qui sera modifiée
                  Intersect (VectorLayer): Couche servant pour la comparaison
                  Méthode (int): 0 = Nouvelle sélection, 1 = Ajouter à la sélection, 2 = Sélection dans la sélection, 3 = Retirer de la sélection
        Fonction : Sélection par localisation grâce aux entités sélectionnées d'une couche pour obtenir les entités à la géoétrie identique
        """
        Params={'INPUT':Entree,
                'PREDICATE':[3],
                'INTERSECT':Intersect,
                'METHOD':Methode}
        processing.run("native:selectbylocation",Params)


    def Extremites(self,Entree):
        """
        Entrées : self
                  Entree (VectorLayer): Couche dont les extrémités des entités doivent être sélectionnés
        Fonction : Récupération du premier et dernier point des lignes d'une couche
        Sorties : Premier (VectorLayer): Premier point
                  Dernier (VectorLayer): Dernier point
        """
        Params ={'INPUT':Entree,
                 'VERTICES':'0',
                 'OUTPUT':'TEMPORARY_OUTPUT'}
        resulttemps=processing.run("native:extractspecificvertices",Params)
        Premier = resulttemps['OUTPUT']

        Params ={'INPUT':Entree,
                 'VERTICES':'-1',
                 'OUTPUT':'TEMPORARY_OUTPUT'}
        resulttemps=processing.run("native:extractspecificvertices",Params)
        Dernier = resulttemps['OUTPUT']

        return Premier, Dernier