"""
Fonctions qualifiables comme utilitaires, utilisables par les différents outils du module
"""
import os

from qgis import *
from qgis.PyQt import *
from qgis.core import *
from PyQt5.QtGui import *
from PyQt5.QtCore import *
from processing.tools import dataobjects
import processing

# This loads your .ui file so that PyQt can populate your plugin with the elements from Qt Designer
path=os.path.dirname(os.path.abspath(__file__))
path=path.replace("Fonctions",'Siligites_dialog_base.ui')
FORM_CLASS, _ = uic.loadUiType(path)

class Utilitaires(QtWidgets.QDialog, FORM_CLASS) :
    def __init__(self, parent=None):
        """Constructor."""
        super(Utilitaires, self).__init__(parent)
        # Set up the user interface from Designer through FORM_CLASS.
        # After self.setupUi() you can access any designer object by doing
        # self.<objectname>, and you can use autoconnect slots - see
        # http://qt-project.org/doc/qt-4.8/designer-using-a-ui-file.html
        # #widgets-and-dialogs-with-auto-connect
        self.setupUi(self)


    def Progression(Barre,Label,Message,Pourcent):
        """
        Entrée : Barre (QgsWidget): Barre de chargement à mettre à jour, 
                 Label (QgsWidget): Zone de texte à modifier pour indiquer les informations de la progression
                 Message (str): Message à afficher, 
                 Pourcent (int): Poucentage d'avancement
        Fonction : Mise à jour d'une barre de chargement en fonction des étapes
        """
        # Remise à 0 de la barre pour faciliter les aller-retours dans les étapes
        Barre.setValue(0)
        # Changement du message lié à la barre de progression
        Label.setText(Message)
        # Fonction pour faire avancer la barre de chargement
        completed=0
        while completed < Pourcent:
            completed += 1
        Barre.setValue(completed)


    def Add_Layers():
        """
        Fonction : Ajout automatique de couches vectorielles en lien avec l'utilisation du module
        """
        n=1
        liste_noms=["Communes","Departements","Regions_Naturelles"]
        for nom in liste_noms :
            # Vérifier si une des couches n'est pas déjà chargée
            if not QgsProject.instance().mapLayersByName(str(n)+" - "+ nom):
                # Accès à la couche dans le dossier du module
                path=os.path.dirname(os.path.abspath(__file__))
                path=path.replace('Fonctions','Couches//{}.shp'.format(nom))
                layer = QgsVectorLayer(path,str(n)+" - "+ nom, "ogr")
                # Définition du style de la couche
                style_path = os.path.dirname(os.path.abspath(__file__))
                style_path=style_path.replace('Fonctions','Styles//{}.qml'.format(nom))
                layer.loadNamedStyle(style_path)
                # Ajout de la couche à la carte
                QgsProject.instance().addMapLayer(layer)
                n=n+1


    def Recherche_Layer(ComboBox):
        """
        Entrée (QgsWidget): Liste déroulante où sera récupéré de la couche recherchée
        Fonction : Sélectionner une couche à partir du nom sélectionné dans une liste déroulante
        """
        nom=ComboBox.currentText()
        # Sélectionner la couche en fonction de son nom
        couche=QgsProject.instance().mapLayersByName(nom)[0]
        return couche
    

    def Prepare_WFS(self,Nom):
        """
        Entrée : Nom (str): Nom du département permettant de charger le flux WFS du GDR SILEX correspondant
        Fonction : Chargement de flux WFS du GDR SILEX en fonction d'un nom du département
        """
        # print("Task") # debugtest
        # Correction et uniformisation des noms de départements pour les utiliser dans un url
        nom_corr=Nom.replace("-","_").replace(" ","_").replace("è","%C3%A8").replace("é","%C3%A9").replace("ô","%C3%B4")
        # print(nom_corr) # debudtest
        # Préparation des paramètres de chargement du flux WFS
        uri = QgsDataSourceUri()
        url = 'https://services3.arcgis.com/zy1UGXGNCAGm516O/arcgis/rest/services/Formations_{}/FeatureServer/0'.format(nom_corr)
        # print(url) # debudtest
        uri.setParam('url', url)
        uri.setSrid('EPSG:3857')
        layer = QgsVectorLayer(uri.uri(), "{}".format(Nom) , 'arcgisfeatureserver')
        if layer.isValid():
            # Ajout de la couche à la carte
            QgsProject.instance().addMapLayer(layer)
            # Création d'un index spatial pour faciliter les recherches par localisation
            layer.dataProvider().createSpatialIndex()
            # Ne pas afficher les entités de la couche pour limiter les calcules
            QgsProject.instance().layerTreeRoot().findLayer(layer.id()).setItemVisibilityChecked(False)
            # Définition du style de la couche
            style_path = os.path.dirname(os.path.abspath(__file__))
            style_path=style_path.replace('Fonctions','Styles//Legende_CODE_HEXA_ETAGE_FINAL_vEV.qml')
            if os.path.exists(style_path):
                layer.loadNamedStyle(style_path)
            else:
                pass
        else:
            print('Invalid layer: failed to add layer') # debudtest

    
    def Creation_Couche(self,Type,EPSG,Nom,Reference):
        """
        Entrée : Type (str): Type de vecteur (Point, LineString, Polygon)
                 EPSG (str): Système de projection de la couche 
                 Nom (str): Nom de la couche agrégé de sortie
                 Reference (VectorLayer): Couche servant de modèle pour colonnes de la table attributaire
        Fonction : Création d'une couche à partir d'un modèle de référence
        Sortie : Couche créée
        """
        # Création de la couche
        Couches_creee=QgsVectorLayer("{}?crs=epsg:{}".format(Type,EPSG), Nom, "memory")
        Layer_data = Couches_creee.dataProvider()
        # Récupération des colonnes de la table attributaire de la couche de référence
        attr = Reference.dataProvider().fields().toList()
        # Mise à jour des colonnes de la nouvelle couche
        Layer_data.addAttributes(attr)
        Couches_creee.updateFields()
        # Ajout de la couche à la carte
        QgsProject.instance().addMapLayer(Couches_creee)

        return Couches_creee
    

    def PlusProche(self,Reférence,Noeuds):
        """
        Entrées : Reference (QGSfeature): Point de référence 
                  Noeuds (VectorLayer): Ensemble de points à comparer
        Fonction : Récupérer le noeud le plus proche d'un réseau par rapport à un point de référence
        Sortie : Point le plus proche de celui de référence
        """
        xa=Reférence.geometry().asPoint()[0]
        ya=Reférence.geometry().asPoint()[1]

        # Création de valeur pour initialiser la recherche
        dist_min=0

        # Recherche dans l'ensemble des sommets du polygone transformé
        if isinstance(Noeuds,list):
            for Sommet in Noeuds :
                # Récupération des coordonnées du sommet
                xb=Sommet.geometry().asPoint()[0]
                yb=Sommet.geometry().asPoint()[1]
                # Calcule de la distance séparant le sommet de l'origine du LCP
                dist=(((xa-xb)**2)+((ya-yb)**2))**0.5
                # Vérification s'il est le plus loin ou le plus proche de l'origine du LCP
                if dist < dist_min or dist_min==0 :
                    Feat_min=Sommet
                    dist_min=dist
        else:
            for Sommet in Noeuds.getFeatures() :
                # Récupération des coordonnées du sommet
                xb=Sommet.geometry().asPoint()[0]
                yb=Sommet.geometry().asPoint()[1]
                # Calcule de la distance séparant le sommet de l'origine du LCP
                dist=(((xa-xb)**2)+((ya-yb)**2))**0.5
                # Vérification s'il est le plus loin ou le plus proche de l'origine du LCP
                if dist < dist_min or dist_min==0 :
                    Feat_min=Sommet
                    dist_min=dist
        return Feat_min
    

    def PlusLoin(self,Reférence,Noeuds):
        """
        Entrées : Reference (QGSfeature): Point de référence 
                  Noeuds (VectorLayer): Ensemble de points à comparer
        Fonction : Récupérer le noeud le plus éloigné d'un réseau par rapport à un point de référence
        Sortie : Point le plus éloigné de celui de référence
        """
        xa=Reférence.geometry().asPoint()[0]
        ya=Reférence.geometry().asPoint()[1]

        # Création de valeur pour initialiser la recherche
        dist_max=0

        # Recherche dans l'ensemble des sommets du polygone transformé
        if isinstance(Noeuds,list):
            for Sommet in Noeuds :
                # Récupération des coordonnées du sommet
                xb=Sommet.geometry().asPoint()[0]
                yb=Sommet.geometry().asPoint()[1]
                # Calcule de la distance séparant le sommet de l'origine du LCP
                dist=(((xa-xb)**2)+((ya-yb)**2))**0.5
                # Vérification s'il est le plus loin ou le plus proche de l'origine du LCP
                if dist > dist_max or dist_max==0 :
                    Feat_max=Sommet
                    dist_max=dist
        else:
            for Sommet in Noeuds.getFeatures() :
                # Récupération des coordonnées du sommet
                xb=Sommet.geometry().asPoint()[0]
                yb=Sommet.geometry().asPoint()[1]
                # Calcule de la distance séparant le sommet de l'origine du LCP
                dist=(((xa-xb)**2)+((ya-yb)**2))**0.5
                # Vérification s'il est le plus loin ou le plus proche de l'origine du LCP
                if dist > dist_max or dist_max==0 :
                    Feat_max=Sommet
                    dist_max=dist
        return Feat_max