# -*- coding: utf-8 -*-
"""
/***************************************************************************
 Silanalyses
                                 A QGIS plugin
 Plugin pour l'analyse de réseaux issus, ou non, de l'outil SilLCP
 Generated by Plugin Builder: http://g-sherman.github.io/Qgis-Plugin-Builder/
                             -------------------
        begin                : 2024-06-03
        git sha              : $Format:%H$
        copyright            : (C) 2024 by Thomas ANDRE
        email                : thomas.andre.archgeo@gmail.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
"""

import os

from qgis import *
from qgis.PyQt import *
from qgis.core import *
from PyQt5.QtGui import *
from PyQt5.QtCore import *
from processing.tools import dataobjects
from distutils.dir_util import copy_tree
from processing.tools.system import mkdir, userFolder
import processing
from math import *
from .Fonctions.Utilitaires import *
from .Fonctions.Fonctions_QGIS import *


# This loads your .ui file so that PyQt can populate your plugin with the elements from Qt Designer
FORM_CLASS, _ = uic.loadUiType(os.path.join(
    os.path.dirname(__file__), 'Silanalyses_dialog_base.ui'))


class SilanalysesDialog(QtWidgets.QDialog, FORM_CLASS):
    def __init__(self, parent=None):
        """Constructor."""
        super(SilanalysesDialog, self).__init__(parent)
        # Set up the user interface from Designer through FORM_CLASS.
        # After self.setupUi() you can access any designer object by doing
        # self.<objectname>, and you can use autoconnect slots - see
        # http://qt-project.org/doc/qt-4.8/designer-using-a-ui-file.html
        # #widgets-and-dialogs-with-auto-connect
        self.setupUi(self)

        # Initialisation de certaines fonctions et outils
        self.Liste_Couches()
        self.Choix_Reseau.setFilters(QgsMapLayerProxyModel.LineLayer)
        self.Parcours.setFilters(QgsMapLayerProxyModel.LineLayer)
        self.Depart.setFilters(QgsMapLayerProxyModel.PointLayer)
        self.Inter.setFilters(QgsMapLayerProxyModel.PointLayer)
        self.Arrivee.setFilters(QgsMapLayerProxyModel.PointLayer)
        self.Inter.setAllowEmptyLayer(True)
        self.Elem_Depart.setAllowNull(True)
        self.Elem_Inter.setAllowNull(True)
        self.Elem_Arrivee.setAllowNull(True)
        
        # Couches_vide=QgsVectorLayer("Point?crs=epsg:2154","Vide", "memory")
        # self.Inter.setAdditionalLayers([Couches_vide])
        self.Inter.setCurrentIndex(0) 

        # Initialisation de la barre de chargement
        Message="Avancement dans l'utiliation des outils"
        Pourcent=0
        Utilitaires.Progression(self.Bar_2,self.message_progres,Message,Pourcent)
        
        # Protection des boutons pour éviter les erreurs
        self.Choix_Champ.setEnabled(False)
        self.OK_Champ.setEnabled(False)
        self.Bouton_temps.setEnabled(False)
        self.Bouton_km.setEnabled(False)
        self.Bouton_autre.setEnabled(False)
        self.jour.setEnabled(False)
        self.heure.setEnabled(False)
        self.minute.setEnabled(False)
        self.km.setEnabled(False)
        self.autre.setEnabled(False)
        self.OK_Resume.setEnabled(False)

        self.jour.hide()
        self.heure.hide()
        self.minute.hide()
        self.km.hide()
        self.autre.hide()
        self.Elem_Inter.hide()

        # Lien entre toutes les fonctions et les boutons
        self.OK_Reseau.clicked.connect(self.Select_Reseau)
        self.OK_Champ.clicked.connect(self.Select_Champ)
        self.Bouton_temps.toggled.connect(self.Select_Unite)
        self.Bouton_km.toggled.connect(self.Select_Unite)
        self.Bouton_autre.toggled.connect(self.Select_Unite)
        self.OK_Resume.clicked.connect(self.Resume)
        self.OK_Nouveau.clicked.connect(self.Fusion)
        self.Refresh.clicked.connect(self.Liste_Couches)
        self.OK_Chemin.clicked.connect(self.Itineraire)
        self.Depart.layerChanged.connect(self.Prepa_listes)
        self.Inter.layerChanged.connect(self.Prepa_listes)
        self.Arrivee.layerChanged.connect(self.Prepa_listes)


        path=os.path.dirname(os.path.abspath(__file__))
        path=path+'//Couches//T//'
        if os.path.isfile(path+'t1.shp') or os.path.isfile(path+'t2.shp'):
            files = os.listdir(path)
            for f in files:
                os.remove(path + f) 
 

    def Prepa_listes(self):
        if self.Depart.currentLayer()!=None:
            dep=self.Depart.currentLayer()
            self.Elem_Depart.setLayer(dep)
            self.Elem_Depart.update()
        if self.Inter.currentLayer()!=None:
            inter=self.Inter.currentLayer()
            self.Elem_Inter.setLayer(inter)
            self.Elem_Inter.show()
        else:
            self.Elem_Inter.hide()
        if self.Arrivee.currentLayer()!=None:
            fin=self.Arrivee.currentLayer()
            self.Elem_Arrivee.setLayer(fin)
        

    def Liste_Couches(self):
        """
        Fonction : Récupération des couches du projet pour rafraichir la liste et ne proposer que les couches de vecteurs
        """
        # Remise à zéro de la liste
        self.Multi_Reseau.clear()
        # Récupération des couches du projet
        self.layers=QgsProject.instance().mapLayers().values()
        for couche in self.layers :
            # Tri des couches pour ne garder que les couches de vecteurs
            if couche.type() == QgsMapLayer.VectorLayer :
                self.Multi_Reseau.addItemWithCheckState(str(couche.name()),False)


    def Reindex(self,index,liste,n):
        """
        Entrées : index (str): Nom de la colonne servant d'index
                  liste (liste de QGSfeatures): Liste des éléments à réindexer
                  n (int): Index à partir duquel commencer
        Fonction : Changer la colonne d'index des éléments sélectionner pour éviter les problèmes de doublons
        Sortie : Liste des éléments réindexés
        """
        # Changement de l'index de chaque entité de la liste
        for feat in liste :
            feat[index]=n
            n+=1
            # print(str(n)) #debugtest

        return liste


    def Fusion(self):
        """
        Fonction : Outil de fusion de différents réseau en 1 seul harmonisé
        """
        # Récupération des couches sélectionnées
        Liste_couche=self.Multi_Reseau.checkedItems()
        # Récupération du nom donné
        Nom=self.Nom_Reseau.text()
        if Nom=='':
            Nom='Reseau_fusion'

        #Création d'une couche de fusion à partir d'une couche de référence
        ref=Liste_couche[0]
        couche=QgsProject.instance().mapLayersByName(ref)[0]
        SCR=couche.crs().authid().replace("EPSG:","")
        Couche_fusion=Utilitaires.Creation_Couche(self,"LineString",SCR,Nom,couche)
        Layer_data = Couche_fusion.dataProvider()
        # print(str(couche.fields().names())) #debugtest

        # Récupération du nom de la couche d'identification des entités pour le modifier et éviter les doublons au moment de la fusion
        self.index=1
        for elem in Liste_couche :
            couche=QgsProject.instance().mapLayersByName(elem)[0]
            couche.selectAll()

            Fonctions_QGIS.Select_Egal(self,couche,Couche_fusion,3)

            Liste_elem =couche.selectedFeatures()

            #Changement des ID des couches pour éviter les problèmes de doublons
            liste_col=couche.fields().names()
            # print(str(liste_col)) #debugtest

            if "ID" in liste_col:
                index="ID"
                index_liste=self.Reindex(index,Liste_elem,self.index)
            if "id" in liste_col:
                index="id"
                index_liste=self.Reindex(index,Liste_elem,self.index)
            if "fid" in liste_col:
                index="fid"
                index_liste=self.Reindex(index,Liste_elem,self.index)

            Layer_data.addFeatures(index_liste)
            self.index=self.index+len(Liste_elem)
            couche.removeSelection()

        QgsProject.instance().addMapLayer(Couche_fusion)
        style_path = os.path.dirname(os.path.abspath(__file__))+'//Styles//Liens.qml'
        if os.path.exists(style_path):
            Couche_fusion.loadNamedStyle(style_path)
        else:
           pass
        self.Nom_Reseau.clear()


    def Select_Reseau(self):
        """
        Fonction : Sélection de la couche contenant le réseau à simplifier
        """
        # Recherche de la couche correcpondant au nom donné
        self.Reseau=Utilitaires.Recherche_Layer(self.Choix_Reseau)
        # Définition de la source de la liste déroulante des entités de la couche sélectionnée
        self.Choix_Champ.setLayer(self.Reseau)
        self.Choix_Champ.setEnabled(True)
        self.OK_Champ.setEnabled(True)


    def Select_Champ(self):
        """
        Fonction : Sélection d'un champ d'attribut pour la sélection du point d'origine des LCP
        """
        self.Champ=self.Choix_Champ.currentField()
        self.Bouton_temps.setEnabled(True)
        self.Bouton_km.setEnabled(True)
        self.Bouton_autre.setEnabled(True)
        self.Bouton_temps.setChecked(True)
        self.jour.setEnabled(True)
        self.heure.setEnabled(True)
        self.minute.setEnabled(True)
        self.OK_Resume.setEnabled(True)

        self.jour.show()
        self.heure.show()
        self.minute.show()


    def Select_Unite(self):
        """
        Fonction : Choix de l'unité utilisé pour les analyses
        """
        if self.Bouton_temps.isChecked():
            self.Bouton_km.setChecked(False)
            self.Bouton_autre.setChecked(False)

            self.jour.setEnabled(True)
            self.heure.setEnabled(True)
            self.minute.setEnabled(True)
            self.jour.show()
            self.heure.show()
            self.minute.show()

            self.Choix=1
            self.OK_Resume.setEnabled(True)

            self.km.setEnabled(False)
            self.autre.setEnabled(False)
            self.km.hide()
            self.autre.hide()

            self.jour.setText("0")
            self.heure.setText("0")
            self.minute.setText("0")
            self.km.setText("0")
            self.autre.setText("0")

        elif self.Bouton_km.isChecked():
            self.Bouton_temps.setChecked(False)
            self.Bouton_autre.setChecked(False)

            self.km.setEnabled(True)
            self.km.show()

            self.Choix=2
            self.OK_Resume.setEnabled(True)

            self.jour.setEnabled(False)
            self.heure.setEnabled(False)
            self.minute.setEnabled(False)
            self.autre.setEnabled(False)
            self.jour.hide()
            self.heure.hide()
            self.minute.hide()
            self.autre.hide()

            self.jour.setText("0")
            self.heure.setText("0")
            self.minute.setText("0")
            self.km.setText("0")
            self.autre.setText("0")

        elif self.Bouton_autre.isChecked():
            self.Bouton_temps.setChecked(False)
            self.Bouton_km.setChecked(False)

            self.autre.setEnabled(True)
            self.autre.show()

            self.Choix=3
            self.OK_Resume.setEnabled(True)

            self.jour.setEnabled(False)
            self.heure.setEnabled(False)
            self.minute.setEnabled(False)
            self.km.setEnabled(False)
            self.jour.hide()
            self.heure.hide()
            self.minute.hide()
            self.km.hide()

            self.jour.setText("0")
            self.heure.setText("0")
            self.minute.setText("0")
            self.km.setText("0")
            self.autre.setText("0")

    
    def Resume(self):
        """
        Fonction : Sélection des lien du réseau à conserver en fonction de la valeur choisie
        """
        Message="Lancement de l'outil de simplification du réseau"
        Pourcent=10
        Utilitaires.Progression(self.Bar_2,self.message_progres,Message,Pourcent)

        self.Reseau.selectAll()
        list_features=[]
        if self.Choix==1:
            Message="Tri des lien en fonction du temps..."
            Pourcent=25
            Utilitaires.Progression(self.Bar_2,self.message_progres,Message,Pourcent)

            for feature in self.Reseau.selectedFeatures():
                # Utilisation de la fonction spécifique au tri de temps
                new_liste=self.Choix_Temps(feature,self.Champ,list_features)
    
        elif self.Choix==2:
            Message="Tri des lien en fonction de la distance..."
            Pourcent=25
            Utilitaires.Progression(self.Bar_2,self.message_progres,Message,Pourcent)
            
            for feature in self.Reseau.selectedFeatures():
                # Utilisation de la fonction spécifique au tri de distance
                new_liste=self.Choix_Dist(feature,self.Champ,list_features)
            
        elif self.Choix==3:
            Message="Tri des lien en fonction du coût..."
            Pourcent=25
            Utilitaires.Progression(self.Bar_2,self.message_progres,Message,Pourcent)

            for feature in self.Reseau.selectedFeatures():
                # Utilisation de la fonction spécifique au tri de cout
                new_liste=self.Choix_Cout(feature,self.Champ,list_features)

        # Création de la couche
        Simplifie=QgsVectorLayer("LineString?crs=epsg:2154", "Reseau_simplifie", "memory")
        Layer_data = Simplifie.dataProvider()

        #Vérification si des entités correspondent bien à la recherche
        Message="Ajout des entités sélectionnées dans une nouvelle couche"
        Pourcent=50
        Utilitaires.Progression(self.Bar_2,self.message_progres,Message,Pourcent)

        if len(new_liste)>0 :
            # Récupération des colonnes de la table attributaire de la couche de référence
            attr = self.Reseau.dataProvider().fields().toList()
            # Mise à jour des colonnes de la nouvelle couche
            Layer_data.addAttributes(attr)
            Simplifie.updateFields()
            # Ajout de la couche à la carte
            QgsProject.instance().addMapLayer(Simplifie)
            style_path = os.path.dirname(os.path.abspath(__file__))+'//Styles//Liens.qml'
            if os.path.exists(style_path):
                Simplifie.loadNamedStyle(style_path)
            else:
                pass
            
            #Ajout des liens sélectionnés à la nouvelle couche
            Layer_data.addFeatures(new_liste)

            Message="Calcule de la centralité du réseau simplifié"
            Pourcent=75
            Utilitaires.Progression(self.Bar_2,self.message_progres,Message,Pourcent)

            # Calcule de la centralité du nouveau réseau
            # print("start") #debugtest
            Centralite=Fonctions_QGIS.Centralite(self,Simplifie)
            QgsProject.instance().addMapLayer(Centralite)
            # Définition du style de la couche
            style_path = os.path.dirname(os.path.abspath(__file__))+'//Styles//Noeuds.qml'
            if os.path.exists(style_path):
                Centralite.loadNamedStyle(style_path)
            else:
                pass
            # print("end") #debugtest

        self.Bouton_temps.setChecked(True)
        self.Bouton_temps.setEnabled(False)
        self.jour.setText("0")
        self.heure.setText("0")
        self.minute.setText("0")
        self.jour.setEnabled(False)
        self.heure.setEnabled(False)
        self.minute.setEnabled(False)
        self.jour.hide()
        self.heure.hide()
        self.minute.hide()
        self.Bouton_km.setEnabled(False)
        self.km.setText("0")
        self.km.setEnabled(False)
        self.km.hide()
        self.Bouton_autre.setEnabled(False)
        self.autre.setText("0")
        self.autre.setEnabled(False)
        self.autre.hide()
        
            
            

        self.Choix_Champ.setEnabled(False)
        self.OK_Champ.setEnabled(False)
        self.OK_Resume.setEnabled(False)
        self.Reseau.removeSelection()

        Message="Avancement dans l'utiliation des outils"
        self.message_progres.setText(Message)
        self.Bar_2.reset()


    def Choix_Temps(self,feature,champ,liste):
        """
        Entrées : feature (QGSfeature): Elément de la couche à vérifier
                  champ (str): Champ où se trouve la valeur à comparer
                  liste (list): Liste à remplir avec les éléments souhaités
        Fonction : Vérifier si le temps lié au lien est inférieur à celui recherché pour le sélectionner
        Sortie : Liste avec les liens sélectionnés
        """
        # Récupération de la valeur du lien
        val=feature[champ]
        # print(str(val)) #debugtest
        
        # Récupértion des valeurs rentrées
        if self.jour.text()!="":
            j = int(self.jour.text())
        else : 
            j=0
        if self.heure.text()!="":
            h = int(self.heure.text())
        else : 
            h=0
        if self.minute.text()!="": 
            m = int(self.minute.text())
        else : 
            m=0

        # print("a {},{},{}".format(j,h,m)) #debugtest

        # Tri du lien en fonction de la valeur de référence
        # Identifiaction des valeurs de temps à partir d'une chaîne de caractères
        if isinstance(val, str) :
            # Format "--j --h --min"
            if "j" in val : 
                jour=int(val.split("j")[0])
                heure=int(val.split("j")[1].split("h")[0])
                min=int(val.split("j")[1].split("h")[1].split("min")[0])

                # print("b {},{},{}".format(jour,heure,min)) #debugtest
                if jour < j:
                    liste.append(feature)
                elif jour == j :
                    if heure < h :
                        liste.append(feature)
                    elif heure == h :
                        if min <= m :
                            liste.append(feature)
                        else :
                            pass
                    else :
                        pass
                else :
                    pass
            else :
                # Format "--h --min"
                heure=int(val.split("h")[0])
                min=int(val.split("h")[1].split("min")[0])
                # print("c {},{}".format(heure,min)) #debugtest
                h=h+j*24
                # print(h) #debugtest

                if heure < h :
                    liste.append(feature)
                elif heure == h :
                    if min <= m :
                        liste.append(feature)
                    else :
                        pass
                else :
                    pass
        # Identifiaction des valeurs de temps à partir d'un nombre entier ou décimal
        elif isinstance(val,float) :
            heure=floor(val)
            min=val-heure
            h=h+j*24
            # print(h) #debugtest
            # print("d {},{}".format(heure,min)) #debugtest

            if heure < h :
                liste.append(feature)
            elif heure == h :
                if min <= m :
                    liste.append(feature)
        else :
            pass
        
        # print(str(len(liste))) #debugtest
        return liste


    def Choix_Dist(self,feature,champ,liste):
        """
        Entrées : feature (QGSfeature): Elément de la couche à vérifier
                  champ (str): Champ où se trouve la valeur à comparer
                  liste (list): Liste à remplir avec les éléments souhaités
        Fonction : Vérifier si la distance liée au lien est inférieur à celui recherché pour le sélectionner
        Sortie : Liste avec les liens sélectionnés
        """
        # Récupération de la valeur du lien
        val=float(feature[champ])
        # print(str(val)) #debugtest

        # Récupération de la valeur rentrée
        ref = float(self.km.text())
        # print(str(ref)) #debugtest

        # Tri du lien en fonction de la valeur de référence
        if val<ref :
            liste.append(feature)
        else :
            pass

        # print(str(len(liste))) #debugtest
        return liste


    def Choix_Cout(self,feature,champ,liste):
        """
        Entrées : feature (QGSfeature): Elément de la couche à vérifier
                  champ (str): Champ où se trouve la valeur à comparer
                  liste (list): Liste à remplir avec les éléments souhaités
        Fonction : Vérifier si le coût lié au lien est inférieur à celui recherché pour le sélectionner
        Sortie : Liste avec les liens sélectionnés
        """
        # Récupération de la valeur du lien
        val=float(feature["{}".format(champ)])
        # print(str(val)) #debugtest
    
        # Récupération de la valeur rentrée
        ref = float(self.autre.text())
        # print(str(ref)) #debugtest

        # Tri du lien en fonction de la valeur de référence
        if val<ref :
            liste.append(feature)
        else :
            pass

        # print(str(len(liste))) #debugtest
        return liste
    

    def Itineraire(self):
        """
        Fonction : Création d'un itinéraire entre 2 ou plusieurs points en empruntant le chemin le plus court d'un réseau
        """
        Message="Préparation du réseau"
        Pourcent=5
        Utilitaires.Progression(self.Bar_2,self.message_progres,Message,Pourcent)

        reseau=Utilitaires.Recherche_Layer(self.Parcours)
        # print(str(reseau)) #debugtest
        # Agrégation du réseau en une entité pour pouvoir utiliser l'outil de chemin le plus court entre deux points
        reseau=Fonctions_QGIS.Agregation_simple(self,reseau)

        Message="Récupération du point de départ"
        Pourcent=10
        Utilitaires.Progression(self.Bar_2,self.message_progres,Message,Pourcent)

        depart=Utilitaires.Recherche_Layer(self.Depart)

        Message="Récupération des points intermédiaires"
        Pourcent=20
        Utilitaires.Progression(self.Bar_2,self.message_progres,Message,Pourcent)

        # print(str(depart)) #debugtest
        if self.Inter.currentText() != "":
            inter=Utilitaires.Recherche_Layer(self.Inter)
        else :
            inter=""
        # print(str(inter)) #debugtest

        Message="Récupération du point d'arrivée"
        Pourcent=30
        Utilitaires.Progression(self.Bar_2,self.message_progres,Message,Pourcent)

        arrivee=Utilitaires.Recherche_Layer(self.Arrivee)
        # print(str(arrivee)) #debugtest

        Message="Récupération des nœuds du réseau"
        Pourcent=40
        Utilitaires.Progression(self.Bar_2,self.message_progres,Message,Pourcent)

        # Récupération des points du réseau au cas où les points sélectionnés ne soient pas directement sur le réseau
        # print("start") #debugtest
        Noeuds=Fonctions_QGIS.Noeuds(self,reseau)
        # QgsProject.instance().addMapLayer(Nœuds) #debugtest
        # print("end") #debugtest

        # Outil de vérification en cas de problème avec le point de départ ou d'arrivée
        PB=False

        # Recherche des nœuds du réseau les plus proches des points du trajet
        if self.Inter.currentText() != "":
            Coords_D,Coords_A,liste_inter_tri,PB=self.Recherche_Extremites(Noeuds,depart,inter,arrivee,PB)
        else :
            Coords_D,Coords_A,PB=self.Recherche_Extremites(Noeuds,depart,inter,arrivee,PB)
            

        if not PB :
            Message="Recherche du chemin le plus court entre les points sélectionnés"
            Pourcent=75
            Utilitaires.Progression(self.Bar_2,self.message_progres,Message,Pourcent)

            # Recherche du chemin le plus court au sein du réseau agrégé entre les différents points du parcours
            if self.Inter.currentText() == "":
                Chemin=Fonctions_QGIS.Chemin(self,reseau,Coords_D,Coords_A)
                # print("Chemin OK") #debugtest
                # QgsProject.instance().addMapLayer(Chemin) # debugtest

                # Création ou sélection d'une couche pour stocker les trajets créés
                if not QgsProject.instance().mapLayersByName("Trajet") :
                    SCR=reseau.crs().authid().replace("EPSG:","")
                    Plus_Court= Utilitaires.Creation_Couche(self,"LineString",SCR,"Trajet",Chemin)
                    # QgsProject.instance().addMapLayer(Plus_Court) # debugtest
                else :
                    Plus_Court=QgsProject.instance().mapLayersByName("Trajet")[0]

                style_path = os.path.dirname(os.path.abspath(__file__))+'//Styles//Trajet.qml'
                if os.path.exists(style_path):
                    Plus_Court.loadNamedStyle(style_path)
                else:
                    pass
                    
                Chemin.selectAll()
                Layer_data = Plus_Court.dataProvider()
                Layer_data.addFeatures(Chemin.selectedFeatures())
            else :
                # print("Lancement tri") #debugtest
                # Rassemblement des points du parcours en une liste
                liste_points=[]
                liste_points.append(Coords_D)
                for feat in liste_inter_tri :
                    liste_points.append(feat)
                liste_points.append(Coords_A)
                # print(str(len(liste_points))) #debugtest

                Message="Recherche du chemin le plus court entre les étapes du trajet..."
                Pourcent=90
                Utilitaires.Progression(self.Bar_2,self.message_progres,Message,Pourcent)

                for n in range(0,len(liste_points)-1):
                    # print(str(n)) #debugtest
                    Chemin=Fonctions_QGIS.Chemin(self,reseau,liste_points[n],liste_points[n+1])
                    # print("Chemin OK ARR") #debugtest
                    # QgsProject.instance().addMapLayer(Chemin) # debugtest

                    # Création ou sélection d'une couche pour stocker les trajets créés
                    if not QgsProject.instance().mapLayersByName("Trajet") :
                        SCR=reseau.crs().authid().replace("EPSG:","")
                        Plus_Court= Utilitaires.Creation_Couche(self,"LineString",SCR,"Trajet",Chemin)
                    else :
                        Plus_Court=QgsProject.instance().mapLayersByName("Trajet")[0]

                    style_path = os.path.dirname(os.path.abspath(__file__))+'//Styles//Trajet.qml'
                    if os.path.exists(style_path):
                        Plus_Court.loadNamedStyle(style_path)
                    else:
                        pass
                    
                    Chemin.selectAll()
                    Layer_data = Plus_Court.dataProvider()
                    Layer_data.addFeatures(Chemin.selectedFeatures())

                    Message="{} chemin(s) sur {} créé(s)...".format(str(n),str(len(liste_points)-1))
                    self.message_progres.setText(Message)  
        else :
            Message="/!\ Problème avec le point de départ ou d'arrivée ! Vérifier qu'un seul point est ou peut être sélectionné"
            self.message_progres.setText(Message)        
            self.Bar_2.reset()

        PB=True
        QgsProject.instance().removeMapLayer(Noeuds)
        reseau.removeSelection()
        self.Inter.setCurrentIndex(0) 
        self.Elem_Inter.setFeature(0)
        depart.removeSelection()
        arrivee.removeSelection()
        self.Elem_Inter.hide()

        Message="Avancement dans l'utiliation des outils"
        self.message_progres.setText(Message)        
        self.Bar_2.reset()


    def Recherche_Extremites(self,Noeuds,depart,inter,arrivee,PB):
        """
        Entrées : Nœuds (Couche de points): Ensemble des nœuds du réseau pour identifier ceux correspondant aux points recherchés
                  depart (Point): Point de départ du trajet
                  inter (Point): Points intermédiaires du trajet
                  arrivee (Point): Point d'arrivée du trajet
                  PB (bool): Vérification si les points de départ et d'arrivée ont été trouvés
        Fonction : Recherche des coordonnées des nœuds du réseau correspondants aux points du trajet si ceux-ci ne sont pas les mêmes
        Sorties : Coords_D (str): Coordonnées du point de départ
                  Coords_A (str): Coordonnées du point d'arrivée
                  liste_inter_tri (list): Liste des coordonnées des points intermédiaires triés
                  PB (bool): Vérification si les points de départ et d'arrivée ont été trouvés
        """
        Message="Localisation du point de départ"
        Pourcent=50
        Utilitaires.Progression(self.Bar_2,self.message_progres,Message,Pourcent)

        # Vérification qu'il n'y ait bien qu'un point sélectionné ou sélectionnable pour le départ
        if self.Elem_Depart.feature().hasGeometry() :
            Ref_D=self.Elem_Depart.feature()
        else :
            if len(depart.selectedFeatures())==0:
                # print(len(depart.selectedFeatures())) #debugtest
                depart.selectAll()
            if len(depart.selectedFeatures())==1:
                Ref_D=depart.selectedFeatures()[0]
            else :
                PB=True
            # print("Départ OK") #debugtest
        # Récupération des coordonnées du nœud du réseau correspondant au point
        Point_Depart=Utilitaires.PlusProche(self,Ref_D,Noeuds)
        Coords_D="{},{} [{}]".format(str(Point_Depart.geometry().asPoint()[0]),str(Point_Depart.geometry().asPoint()[1]),depart.crs().authid())
        # print(Coords_D) #debugtest

        Message="Localisation du point d'arrivée"
        Pourcent=55
        Utilitaires.Progression(self.Bar_2,self.message_progres,Message,Pourcent)

        # Vérification qu'il n'y ait bien qu'un point sélectionné ou sélectionnable pour l'arrivée
        if self.Elem_Arrivee.feature().hasGeometry() :
            Ref_A=self.Elem_Arrivee.feature()
        else :
            if len(arrivee.selectedFeatures())==0:
                # print(len(arrivee.selectedFeatures())) #debugtest
                arrivee.selectAll()
            if len(arrivee.selectedFeatures())==1:
                Ref_A=arrivee.selectedFeatures()[0]
            else :
                PB=True
            # print("Arrivée OK") #debugtest
        # Récupération des coordonnées du nœud du réseau correspondant au point
        Point_Arrivee=Utilitaires.PlusProche(self,Ref_A,Noeuds)
        Coords_A="{},{} [{}]".format(str(Point_Arrivee.geometry().asPoint()[0]),str(Point_Arrivee.geometry().asPoint()[1]),arrivee.crs().authid())
        # print(Coords_A) #debugtest

        # Vérification de la présence de points intermédiaires
        if self.Inter.currentText() != "":
            Message="Localisation et tri des points du trajet"
            Pourcent=50
            Utilitaires.Progression(self.Bar_2,self.message_progres,Message,Pourcent)

        
            liste_inter_tri=[]
            # Vérification si des points sont déjà sélectonnés, ou si l'ensemble de la couche est à prendre en compte
            if self.Elem_Inter.feature().hasGeometry() :
                liste_inter=[]
                liste_tempo=[]
                liste_inter.append(self.Elem_Inter.feature())
                liste_tempo.append(self.Elem_Inter.feature())
            else :
                if len(inter.selectedFeatures())==0:
                    # print(len(inter.selectedFeatures())) #debugtest
                    inter.selectAll()
                    liste_inter=inter.selectedFeatures()
                else :
                    liste_inter=inter.selectedFeatures()
                # Mise en liste des points pour les trier
                liste_tempo=inter.selectedFeatures()

            if not PB :
                # print(str(len(liste_tempo))) #debugtest
                liste_tri=[]
                # Définition du premier point du trajet comme référence
                point_A=Ref_A
                while len(liste_tri)!=len(liste_inter):
                    # Récupération du point précédent
                    Point_Inter=Utilitaires.PlusLoin(self,point_A,liste_tempo)
                    liste_tri.append(Point_Inter)
                    # Suppresion du point de la liste temporaire pour qu'il ne soit pas choisit à nouveau
                    liste_tempo.remove(Point_Inter)
                    # Changement de point de référence

                for feat in liste_tri:
                    # Récupération des coordonnées du nœud du réseau correspondant au point
                    Point_Inter=Utilitaires.PlusProche(self,feat,Noeuds)
                    Coords_I="{},{} [{}]".format(str(Point_Inter.geometry().asPoint()[0]),str(Point_Inter.geometry().asPoint()[1]),inter.crs().authid())
                    liste_inter_tri.append(Coords_I)
            # print("Intermédiaire OK") #debugtest

            return Coords_D,Coords_A,liste_inter_tri,PB
        else :
            return Coords_D,Coords_A,PB