# -*- coding: utf-8 -*-
"""
/***************************************************************************
 Silextracteur
                                 A QGIS plugin
 Plugin pour l'extraction de formations géologiques à silicites depuis les flux WFS du GDR SILEX
 Generated by Plugin Builder: http://g-sherman.github.io/Qgis-Plugin-Builder/
                             -------------------
        begin                : 2024-06-03
        git sha              : $Format:%H$
        copyright            : (C) 2024 by Thomas ANDRE
        email                : thomas.andre.archgeo@gmail.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
"""

import os

from qgis import *
from qgis.PyQt import *
from qgis.core import *
from PyQt5.QtGui import *
from PyQt5.QtCore import *
from processing.tools import dataobjects
import processing
from .Fonctions.Utilitaires import *
from .Fonctions.Fonctions_QGIS import *


# This loads your .ui file so that PyQt can populate your plugin with the elements from Qt Designer
FORM_CLASS, _ = uic.loadUiType(os.path.join(
    os.path.dirname(__file__), 'Silextracteur_dialog_base.ui'))


class SilextracteurDialog(QtWidgets.QDialog, FORM_CLASS):
    def __init__(self, parent=None):
        """Constructor."""
        super(SilextracteurDialog, self).__init__(parent)
        # Set up the user interface from Designer through FORM_CLASS.
        # After self.setupUi() you can access any designer object by doing
        # self.<objectname>, and you can use autoconnect slots - see
        # http://qt-project.org/doc/qt-4.8/designer-using-a-ui-file.html
        # #widgets-and-dialogs-with-auto-connect
        self.setupUi(self)

        # Initialisation de certaines fonctions et outils
        self.layers=QgsProject.instance().mapLayers().values()
        self.champ="etage_final"
        self.Choix_Zone.setFilters(QgsMapLayerProxyModel.PolygonLayer)
        self.Choix_Depart.setFilters(QgsMapLayerProxyModel.PolygonLayer)
        self.Choix_Villes.setFilters(QgsMapLayerProxyModel.PointLayer)
        self.Charge=False

        # Initialisation de la barre de chargement
        Message="Avancement dans la récupération des formations géologiques"
        Pourcent=0
        Utilitaires.Progression(self.Bar_1,self.progres_label,Message,Pourcent)
        self.Bar_1.setValue(Pourcent)

        # Protection des boutons pour éviter les erreurs
        self.Choix_Champ.setEnabled(False)
        self.Liste_Zone.setEnabled(False)
        self.OK_Zone_2.setEnabled(False)
        self.Choix_Admin.setEnabled(False)
        self.OK_Admin.setEnabled(False)
        self.Choix_Depart.setEnabled(False)
        self.OK_Zone_3.setEnabled(False)
        self.Launch.setEnabled(False)
        self.Colonne_Nom.setEnabled(False)
        self.Choix_WFS.setEnabled(False)
        self.OK_Depart.setEnabled(False)
        self.Bouton_Etage.setChecked(True)
        self.Liste_Strati.setEnabled(False)
        self.OK_Strati.setEnabled(False)
        self.Choix_Villes.setEnabled(False)
        self.OK_Villes.setEnabled(False)
        self.Champ_Villes.setEnabled(False)
        self.Liste_Villes.setEnabled(False)
        self.OK_Villes_2.setEnabled(False)
        self.Villes.setEnabled(False)
        self.Half_Launch.setEnabled(False)
        self.Final_Launch.setEnabled(False)

        self.Choix_Depart.hide()
        self.OK_Zone_3.hide()
        self.Colonne_Nom.hide()
        self.Launch.hide()
        self.label_8.hide()
        self.label_15.hide()


        # Lien entre toutes les fonctions et les boutons
        self.Preset.clicked.connect(Utilitaires.Add_Layers)
        self.OK_Zone.clicked.connect(self.Load_Zone)
        self.Choix_Champ.fieldChanged.connect(self.Select_Champ)
        self.OK_Zone_2.clicked.connect(self.Select_Zone)
        self.OK_Admin.clicked.connect(self.Select_Admin)
        self.OK_Zone_3.clicked.connect(self.Select_Depart)
        self.Launch.clicked.connect(self.Load_WFS)
        self.OK_Depart.clicked.connect(self.Select_WFS)
        self.Bouton_Descr.toggled.connect(self.Liste_Types)
        self.Bouton_Etage.toggled.connect(self.Liste_Types)
        self.OK_Strati.clicked.connect(self.Select_Strati)
        self.Villes.toggled.connect(self.Access_Villes)
        self.OK_Villes.clicked.connect(self.Load_Villes)
        self.Champ_Villes.fieldChanged.connect(self.Choix_Champ_Villes)
        self.OK_Villes_2.clicked.connect(self.Select_Villes)
        self.Half_Launch.clicked.connect(self.Recherche_Strati)
        self.Final_Launch.clicked.connect(self.Couche_Finale)
        

        path=os.path.dirname(os.path.abspath(__file__))
        path=path+'//Couches//T//'
        if os.path.isfile(path+'t1.shp') or os.path.isfile(path+'t2.shp'):
            files = os.listdir(path)
            for f in files:
                os.remove(path + f)


    def Access_Villes(self):
        """
        Fonction : Modification des informations et accès du module suivant si l'utilisateur souhaite utiliser une ville ou non
        """
        # Vérification de la volonté d'utiliser une ville pour la sélection
        if self.Villes.isChecked():
            self.Choix_Villes.setEnabled(True)
            self.OK_Villes.setEnabled(True)
            self.Half_Launch.setEnabled(False)

            Message='Sélectionner la couche des communes (6/8)'
            self.progres_label.setText(Message)
        else :
            self.Choix_Villes.setEnabled(False)
            self.OK_Villes.setEnabled(False)
            self.Champ_Villes.setEnabled(False)
            self.Liste_Villes.setEnabled(False)
            self.OK_Villes_2.setEnabled(False)



    def Liste_Types(self):
        """
        Fonction permettant de changer le champ listé dans la liste déroulante lors de sélection de la stratigraphie
        """
        if self.Bouton_Etage.isChecked():
            self.Bouton_Descr.setChecked(False)
            self.champ="etage_final"
        elif self.Bouton_Descr.isChecked():
            self.Bouton_Etage.setChecked(False)
            self.champ="descr"


    def Load_Zone(self):
        """
        Fonction : Permettre la sélection des zones d'extractions une fois la couche sélectionnée
        """
        self.Couche_Zones=Utilitaires.Recherche_Layer(self.Choix_Zone)
        # Définir la source de la liste déroulante des entités de la couche sélectionnée
        self.Choix_Champ.setLayer(self.Couche_Zones)
        self.Liste_Zone.setSourceLayer(self.Couche_Zones)

        self.Choix_Champ.setEnabled(True)
        self.Liste_Zone.setEnabled(True)
        self.OK_Zone_2.setEnabled(True)

        Message="Sélectionnez la zone d'extraction recherchée (1/6)"
        self.progres_label.setText(Message)

    
    def Select_Champ(self):
        """
        Fonction : Sélection d'un champ d'attribut pour la sélection du point d'origine des LCP
        """
        self.Champ_Zones=self.Choix_Champ.currentField()
        # print(str(self.Champ_Zones)) # debugtest
        # Changement de champ de référence pour le choix de l'entité
        self.Liste_Zone.setDisplayExpression(self.Champ_Zones)


    def Select_Zone(self):
        """
        Fonction : Sélectionner la zone d'extraction recherchée
        """
        self.Champ_Zones=self.Choix_Champ.currentField()
        Nom_Zone=self.Liste_Zone.currentText()
        # print(str(Nom_Zone)) # debugtest
        # Sélection dans la couche grâce à une expression donnée
        Fonctions_QGIS.Selectbyattribute(self,self.Couche_Zones,self.Champ_Zones,Nom_Zone,0)

        if self.Charge:
            if self.Choix_Admin.currentText () == "Départements français" :
                self.Choix_Depart.hide()
                self.OK_Zone_3.hide()
                self.Colonne_Nom.hide()
                self.Launch.hide()
                self.label_8.hide()
                self.label_15.hide()
                if not QgsProject.instance().mapLayersByName("2 - Departements"):
                    self.Couche_Depart=QgsProject.instance().mapLayersByName("Departements")[0]
                else:
                    self.Couche_Depart=QgsProject.instance().mapLayersByName("2 - Departements")[0]
            else :
                self.Couche_Depart=Utilitaires.Recherche_Layer(self.Choix_Depart)
            self.Load_WFS()
            Message="Zone d'extraction sélectionnée (2/6)"
            self.progres_label.setText(Message)
        
        self.Choix_Admin.setEnabled(True)
        self.OK_Admin.setEnabled(True)


    def Select_Admin(self):
        if self.Choix_Admin.currentText () == "Départements français" :
            self.Choix_Depart.hide()
            self.OK_Zone_3.hide()
            self.Colonne_Nom.hide()
            self.Launch.hide()
            self.label_8.hide()
            self.label_15.hide()

            if not QgsProject.instance().mapLayersByName("2 - Departements"):
                # Accès à la couche dans le dossier du module
                path=os.path.dirname(os.path.abspath(__file__))
                path=path+'//Couches//{}.shp'.format("Departements")
                layer = QgsVectorLayer(path,"Departements", "ogr")
                # Définition du style de la couche
                style_path = os.path.dirname(os.path.abspath(__file__))
                style_path=style_path+'//Styles//{}.qml'.format("Departements")
                layer.loadNamedStyle(style_path)
                if not QgsProject.instance().mapLayersByName("Departements"):
                    # Ajout de la couche à la carte
                    QgsProject.instance().addMapLayer(layer)

                self.Couche_Depart=QgsProject.instance().mapLayersByName("Departements")[0]

            else :
                self.Couche_Depart=QgsProject.instance().mapLayersByName("2 - Departements")[0]
            
            self.Colonne_D="NOM"
            self.Load_WFS()
        
        if self.Choix_Admin.currentText () == "Personalisée" :
            self.Choix_Depart.show()
            self.OK_Zone_3.show()
            self.Colonne_Nom.show()
            self.Launch.show()
            self.label_8.show()
            self.label_15.show()

            self.Choix_Depart.setEnabled(True)
            self.OK_Zone_3.setEnabled(True)


    def Select_Depart(self):
        """
        Fonction : Sélection de la couche contenant les entités administratives
        """
        self.Couche_Depart=Utilitaires.Recherche_Layer(self.Choix_Depart)
        # Définir la source de la liste déroulante des entités de la couche sélectionnée
        self.Colonne_Nom.setLayer(self.Couche_Depart)

        self.Colonne_Nom.setEnabled(True)
        self.Launch.setEnabled(True)

        Message="Sélectionnez la colonne de Nom des entités administratives (3/6)"
        self.progres_label.setText(Message)


    def Load_WFS(self) :
        """
        Fonction : Chargement du flux WFS du GDR SILEX couvrant l'ensemble de la France,
                   liste des entités administratives concernés et récupération d'un premier échantillon de decription de formations géologiques
        """

        Message="Chargement du Flux WFS..."
        Pourcent=50
        Utilitaires.Progression(self.Bar_1,self.progres_label,Message,Pourcent)

        if self.Choix_Admin.currentText () == "Personalisée" :
            # Récupération de l'intitulé de la colonne de la couche des entités administratives contenant leur Nom
            self.Colonne_D=self.Colonne_Nom.currentField()

        # Création d'une érosion de la zone d'extraction pour sélectionner les entités administratives qu'elle recouvre en enlevant des problèmes précision/simplification des frontières
        buffered_layer = Fonctions_QGIS.Buffer(self,self.Couche_Zones,-2500,True,True)

        if self.Colonne_D == '':
            self.Charge=False

        # Vérifier si une des couches n'est pas déjà chargée
        if not QgsProject.instance().mapLayersByName("WFS_GDR_SILEX") :
            # Préparation des paramètres de chargement du flux WFS
            uri = QgsDataSourceUri()
            url = 'https://services3.arcgis.com/zy1UGXGNCAGm516O/arcgis/rest/services/dpt_formations_silicite_new/FeatureServer/0'
            # print(url) # debudtest
            uri.setParam('url', url)
            uri.setSrid('EPSG:3857')
            self.GDR_SILEX = QgsVectorLayer(uri.uri(), "WFS_GDR_SILEX" , 'arcgisfeatureserver')
            # Ajout de la couche à la carte
            QgsProject.instance().addMapLayer(self.GDR_SILEX)
            # Création d'un index spatial pour faciliter les recherches par localisation
            self.GDR_SILEX.dataProvider().createSpatialIndex()
            # Définition du style de la couche de trajet
            style_path = os.path.dirname(os.path.abspath(__file__))+'//Styles//Legende_CODE_HEXA_ETAGE_FINAL_vEV.qml'
            if os.path.exists(style_path):
                self.GDR_SILEX.loadNamedStyle(style_path)
            # Ne pas afficher les entités de la couche pour limiter les calcules
            QgsProject.instance().layerTreeRoot().findLayer(self.GDR_SILEX.id()).setItemVisibilityChecked(False)

            self.Charge=True

        else :
            # Sélectionner la couche en fonction de son nom
            self.GDR_SILEX= QgsProject.instance().mapLayersByName("WFS_GDR_SILEX")[0]
            # print(str(self.GDR_SILEX.source())) # debugtest
            self.Charge=True

        # Déselection des entités pour faciliter les recherches et éviter les doublons
        self.Couche_Depart.removeSelection()

        # Sélection par localisation des entités administratives recouverts par l'érosion
        Fonctions_QGIS.Selectbylocation_simple(self,self.Couche_Depart,buffered_layer,0)
        # Sélection par localisation des formations géologiques intersectant les entités administratives sélectionnés
        Fonctions_QGIS.Selectbylocation(self,self.GDR_SILEX,self.Couche_Depart,True,0)

        if len(self.GDR_SILEX.selectedFeatures())!=0:
            # Création d'une liste des entités administratives
            self.Choix_WFS.clear()
            self.list_depart=[]
            # Boucle des éléments sélectionnés dans la couche
            for feature in self.Couche_Depart.selectedFeatures():
                # print(feature['{}'.format(self.Colonne_D)]) # debudtest
                nom=feature['{}'.format(self.Colonne_D)]
                # print(nom) # debudtest
                self.list_depart.append(nom)
            # Tri dans la liste
            self.list_depart.sort()
            # Ajout des entités administratives à la liste déroulante
            self.Choix_WFS.addItems(self.list_depart)

            # Création d'une liste des descriptions
            self.Liste_Strati.clear()
            Liste_Descr=[]
            # Réinitialisation de la liste déroulante pour de nouvelles sélections
            self.Liste_Strati.clear()
            self.Liste_Strati.clearEditText()
            # Boucle des éléments sélectionnés dans la couche
            for feature in self.GDR_SILEX.selectedFeatures():
                Liste_Descr.append(feature[self.champ])
            # Récupération des valeurs uniques
            Unique=set(Liste_Descr)
            Liste_Descr=list(Unique)
            # Tri dans la liste
            Liste_Descr.sort()
            # Ajout des descriptions à la liste déroulante
            self.Liste_Strati.addItems(Liste_Descr)

            self.Choix_WFS.setEnabled(True)
            self.OK_Depart.setEnabled(True)
            self.Liste_Strati.setEnabled(True)
            self.OK_Strati.setEnabled(True)

            self.Bar_1.reset()
            Select=len(self.GDR_SILEX.selectedFeatures())
            Message="Préparations terminées (Nb Select : {}) (4/6)".format(Select)
            self.progres_label.setText(Message)

        else :
            self.Bar_1.reset()
            Message="Aucune formation sélectionnable"
            self.progres_label.setText(Message)


    def Select_WFS(self):
        """
        Fonction : Choix du entités administratives recherché et sélection des couches géologiques qui lui sont liées
        """
        Message="Récupération des formations géologiques de l'entité administrative..."
        Pourcent=50
        Utilitaires.Progression(self.Bar_1,self.progres_label,Message,Pourcent)

        Depart=self.Choix_WFS.currentText()
        # print("A : "+Depart) # debugtest

        # Sélection par attribut de l'entité administrative sélectionné dans la couche des entités administratives 
        Fonctions_QGIS.Selectbyattribute(self,self.Couche_Depart,self.Colonne_D,Depart,0)
        # print("B : "+str(len(self.Couche_Depart.selectedFeatures()))) # debugtest
        # Sélection par localisation des formations géologiques couvertes par l'entités administratives sélectionné
        Fonctions_QGIS.Selectbylocation(self,self.GDR_SILEX,self.Couche_Depart,True,0)
        
        # Création d'une liste des descriptions
        Liste_Descr=[]
        # Réinitialisation de la liste déroulante pour de nouvelles sélections
        self.Liste_Strati.clear()
        self.Liste_Strati.clearEditText()
        # print("C : "+str(len(self.GDR_SILEX.selectedFeatures()))) # debugtest

        # Boucle des éléments sélectionnés dans la couche
        for feature in self.GDR_SILEX.selectedFeatures():
            # Création d'une liste pour contenir le Nom de l'entité administrative et sa version en majuscules
            Dep=[]
            d=Depart
            Dep.append(d)
            # Mise en majuscule
            D=d.upper()
            Dep.append(D)
            # print(Dep) # debugtest
            # print(str(feature["nom_dept"])) # debugtest
            # print(str(feature["nom_dept"]) == Dep) # debugtest

            # N'ajouter les description des formations géologiques si elles sont liées à entité administrative sélectionné une seule fois
            if feature[self.champ] not in Liste_Descr and feature["nom_dept"] in Dep :
                Liste_Descr.append(feature[self.champ])

        # Tri dans la liste
        self.Liste_Strati.clear()
        Liste_Descr.sort()
        # Ajout des descriptions à la liste déroulante
        self.Liste_Strati.addItems(Liste_Descr)
        # Déselection des entités pour faciliter les recherches et éviter les doublons
        self.Couche_Depart.removeSelection()

        self.Bar_1.reset()
        Select=len(self.GDR_SILEX.selectedFeatures())
        Message="Sélectionnez la formation géologique recherchée (Nb Select : {}) (5/6)".format(Select)
        self.progres_label.setText(Message)
    

    def Select_Strati(self):
        """
        Fonction : Sélection des formations recherchées à partir de leurs description
        """
        self.Strati=self.Liste_Strati.currentText()

        # print("A: "+str(len(self.GDR_SILEX.selectedFeatures()))) # debudtest
        if len(self.GDR_SILEX.selectedFeatures())!=0:
            # Sélection par attribut des formations géologiques à partir de la description sélectionnée dans la sélection existante
            Fonctions_QGIS.Selectbyattribute(self,self.GDR_SILEX,self.champ,self.Strati,3)
        else : # En cas de chargement d'un second type de formation géologique
            Depart=self.Choix_WFS.currentText()
            # Sélection par attribut de l'entité administrative concerné dans la couche des entités administratives
            Fonctions_QGIS.Selectbyattribute(self,self.Couche_Depart,self.Colonne_D,Depart,0)
            # print("B : "+str(len(self.Couche_Depart.selectedFeatures()))) # debugtest
            # Sélection par localisation des formations géologiques à partir des entités administratives sélectionnés
            Fonctions_QGIS.Selectbylocation(self,self.GDR_SILEX,self.Couche_Depart,True,0)
            # print("C: "+str(len(self.GDR_SILEX.selectedFeatures()))) # debudtest
            # Sélection par attribut des formations géologiques à partir de la description sélectionnée dans la sélection existence
            Fonctions_QGIS.Selectbyattribute(self,self.GDR_SILEX,self.champ,self.Strati,3)
            # Déselection des entités pour faciliter les recherches et éviter les doublons
            # print("D: "+str(len(self.GDR_SILEX.selectedFeatures()))) # debudtest
            self.Couche_Depart.removeSelection()

        self.Half_Launch.setEnabled(True)
        self.Villes.setEnabled(True)

        Select=len(self.GDR_SILEX.selectedFeatures())
        Message="Formation géologique sélectionnée (Nb Select : {}) (6/6)".format(Select)
        self.progres_label.setText(Message)


    def Load_Villes(self):
        """
        Fonction : Chargement de la couche regroupant les noms des villes
        """
        self.Couche_Villes=Utilitaires.Recherche_Layer(self.Choix_Villes)
        # Définir la source de la liste déroulante des entités de la couche sélectionnée
        self.Champ_Villes.setLayer(self.Couche_Villes)
        self.Liste_Villes.setSourceLayer(self.Couche_Villes)

        self.Liste_Villes.setEnabled(True)
        self.Champ_Villes.setEnabled(True)
        self.OK_Villes_2.setEnabled(True)

        Message="Sélectionnez la ville recherchée (7/8)"
        self.progres_label.setText(Message)


    def Choix_Champ_Villes(self) :
        """
        Fonction : Choix du champ d'identification des villes et communes
        """
        self.Liste_Villes.setSourceLayer(self.Couche_Villes)
        self.Champ_V=self.Champ_Villes.currentField()

        # Changement de colonne pour la sélection, cette dernière permettant de différencier des villes portant le même nom
        self.Liste_Villes.setDisplayExpression(" \"{}\" ".format(self.Champ_V))


    def Select_Villes(self):
        """
        Fonction : Sélection de la ville recherchée dans la couche des villes
        """
        self.Champ_V=self.Champ_Villes.currentField()
        print(self.Champ_V)
        Nom_Ville=self.Liste_Villes.currentText()
        print(Nom_Ville)
        # Sélection dans la couche grâce à une expression donnée
        self.Ville=self.Couche_Villes.selectByExpression(" \"{}\" = '{}' ".format(self.Champ_V,Nom_Ville))
        self.Half_Launch.setEnabled(True)

        Message="Ville sélectionnée (8/8)"
        self.progres_label.setText(Message)
        

    def Recherche_Strati(self):
        """
        Fonction : Sélection des formations géologiques suivant le choix de se servir d'une ville ou non
        """
        # Vérification de la volonté d'utiliser une ville pour la sélection
        if self.Villes.isChecked():
            if len(self.Couche_Villes.selectedFeatures())==0:
                self.Select_Villes()
            # Création d'un tampon autour de la ville, correspondant à la distance dans laquelle les formations géologiques ont le plus de chances de se trouver
            self.Buffer=Fonctions_QGIS.Buffer(self,self.Couche_Villes,10000,True,True)
            # Création d'une zone supplémentaire pour éviter que des formations trop éloignées ne soient sélectionnées
            Min=Fonctions_QGIS.Buffer(self,self.Couche_Villes,12000,True,True)
            Max=Fonctions_QGIS.Buffer(self,self.Couche_Villes,30000,True,True)
            self.Limite=Fonctions_QGIS.Difference(self,Min,Max)

            Features=self.GDR_SILEX.selectedFeatures()
            # print("A nb select = "+str(len(Features))) # debugtest
            if len(Features)!=0:
                # Sélection par localisation des formations géologiques à partir du tampon
                Fonctions_QGIS.Selectbylocation_simple(self,self.GDR_SILEX,self.Buffer,2)

            Features=self.GDR_SILEX.selectedFeatures()
            # print("B nb select = "+str(len(Features))) # debugtest
            if len(Features)!=0:
                # Sélection par attributs des formations géologiques possédant la description recherchée dans la sélection réalisée
                Fonctions_QGIS.Selectbyattribute(self,self.GDR_SILEX,self.champ,self.Strati,3)

                # Récupération des éléments sélectionnés dans la couche
                Features=self.GDR_SILEX.selectedFeatures()
                # print("C nb select = "+str(len(Features))) # debugtest

            # Déselection des entités pour faciliter les recherches et éviter les doublons
            self.Couche_Villes.removeSelection()
            self.Couche_Zones.removeSelection()

            Message="Début de la sélection des entités"
            Pourcent=25
            Utilitaires.Progression(self.Bar_1,self.progres_label,Message,Pourcent)

        else:
            # print("A "+str(len(self.Couche_Zones.selectedFeatures()))) #debugtest
            if len(self.Couche_Zones.selectedFeatures())!=0:
                # print("B "+str(len(self.GDR_SILEX.selectedFeatures()))) #debugtest
                buffer=Fonctions_QGIS.Buffer(self,self.Couche_Zones,1000,True)
                # Sélection par localisation des formations géologiques à partir de la zone d'extraction sélectionnée dans la sélection existante
                Fonctions_QGIS.Selectbylocation_simple(self,self.GDR_SILEX,buffer,0)
                # print("C "+str(len(self.GDR_SILEX.selectedFeatures()))) #debugtest
                if len(self.GDR_SILEX.selectedFeatures())!=0:
                    # Sélection par attribut des formations géologiques à partir de la description sélectionnée dans la sélection existence
                    Fonctions_QGIS.Selectbyattribute(self,self.GDR_SILEX,self.champ,self.Strati,3)

            # Récupération des éléments sélectionnés dans la couche
            Features=self.GDR_SILEX.selectedFeatures()
            # print("b_nb select = "+str(len(Features))) # debugtest

            # Déselection des entités pour faciliter les recherches et éviter les doublons
            self.Couche_Zones.removeSelection()

            Select=len(self.GDR_SILEX.selectedFeatures())
            Message="Début de la sélection des entités (Nb Select : {})".format(Select)
            Pourcent=50
            Utilitaires.Progression(self.Bar_1,self.progres_label,Message,Pourcent)

        Nom=self.Nom_Couche.text()
        if Nom=="":
            Nom="Formations_Geol"

        # Vérifier si une des couches n'est pas déjà chargée
        if not QgsProject.instance().mapLayersByName(Nom):
            # Création de la couche intermédiaire qui contient les formations géologiques sélectionnées avant l'agrégation
            Formations_Geol=Utilitaires.Creation_Couche(self,"Polygon","3857",Nom,self.GDR_SILEX)
            # Définition du style de la couche
            style_path = os.path.dirname(os.path.abspath(__file__))+'//Styles//Legende_CODE_HEXA_ETAGE_FINAL_vEV.qml'
            if os.path.exists(style_path):
                Formations_Geol.loadNamedStyle(style_path)
            else:
                pass
        
        # Sélectionner la couche en fonction de son nom
        Formations_Geol=QgsProject.instance().mapLayersByName(Nom)[0]
        # Ajout dans la couche intermédiaire des entités sélectionnées
        Layer_data = Formations_Geol.dataProvider()
        Layer_data.addFeatures(Features)
        
        # Vérification de la volonté d'utiliser une ville pour la sélection
        if self.Villes.isChecked():
            # Utilisation de la fonction de sélection spécifique aux villes
            self.Cas_2(Formations_Geol,Layer_data)
            self.Couche_Depart.removeSelection()
        else :
            # Utilisation de la fonction de sélection spécifique aux zone d'extractions
            self.Cas_1(Formations_Geol,Layer_data)  
            self.Couche_Depart.removeSelection()
        
        Message="Fin de la sélection des couches géologiques. Terminer la sélection ou lancer une nouvelle recherche"
        Pourcent=100
        Utilitaires.Progression(self.Bar_1,self.progres_label,Message,Pourcent)

        # Déselection des entités pour faciliter les recherches et éviter les doublons
        self.GDR_SILEX.removeSelection()

        self.Half_Launch.setEnabled(False)
        self.Final_Launch.setEnabled(True)
        self.Villes.setChecked(False)


    def Boucle_verif(self,Couche,WFS,Champ,Strati):
        """
        Entrées : Couche (VectorLayer): Couche des formations géologiques en cours de complétion
                  WFS (VectorLayer ou flux): Flux WFS où seront sélectionnées les entités
                  Champ (str): Nom du champ de référence pour la sélection
                  Strati (str): Valeur permettant la sélection
        Fonction : Réaliser un tour de sélection d'entités d'un flux WFS pour la couche en cours de complétion
        """
        # print("0") # debugtest
        # Création d'un tampon autour des formations géologiques déjà isolées
        buffered_layer=Fonctions_QGIS.Buffer(self,Couche,1000,False,True)
        # print("1") # debugtest
        # Sélection par localisation des formations géologiques intersectant le tampon dans la sélection déjà existante
        Fonctions_QGIS.Selectbylocation_simple(self,WFS,buffered_layer,0)
        # print("2") # debugtest
        # Sélection par attribut des formations géologiques en foction de la valeur de référence dans la sélection déjà existante
        Fonctions_QGIS.Selectbyattribute(self,WFS,Champ,Strati,3)
        # print("3") # debugtest
        # Création d'une érosion des formations géologiques déjà isolées
        buffered_layer=Fonctions_QGIS.Buffer(self,Couche,-1,False,True)
        # Vérification de la volonté d'utiliser une ville pour la sélection
        if self.Villes.isChecked():
            # Sélection par localisation des formations géologiques intersectant la zone de limite pour les déselectionner
            Fonctions_QGIS.Selectbylocation_simple(self,WFS,self.Limite,3)
        # Sélection par localisation des formations géologiques intersectant l'érosion pour déselectionner les formations géologiques déjà isolées
        Fonctions_QGIS.Selectbylocation_simple(self,WFS,buffered_layer,3)
        # print("4") # debugtest


    def Cas_1(self,Formations_Geol,Layer_data):
        """
        Entrées : Formations_Geol (VectorLayer): Couche intermédiaire qui contient les formations géologiques sélectionnées avant l'agrégation 
                  Layer_data (informations d'une VectorLayer): Accès aux données de la couche en cours de modification
        Fonction : Sélection de formations géologiques complémentaires à partr de la zone d'extraction
        """
        # Récupération des éléments sélectionnés dans la couche
        Features=self.GDR_SILEX.selectedFeatures()
        # print("nb select = "+str(len(Features))) # debugtest
        if len(Features)!=0:
            # Utilisation de la boucle de vérification pour sélectionner les formations complémentaires
            self.Boucle_verif(Formations_Geol,self.GDR_SILEX,self.champ,self.Strati)
        
        # Récupération des éléments sélectionnés dans la couche
        Features=self.GDR_SILEX.selectedFeatures()
        # print("nb select = "+str(len(Features))) # debugtest
        # Vérifier si des entités sont bien sélectionnées
        if len(Features)!=0:
            # Ajouter les entités sélectionnées dans la couche en cours de complétion 
            Layer_data.addFeatures(Features)
            # Récupération de l'étage final des entités sélectionnées pour l'utiliser ensuite pour compléter la sélection
            List_departs=[]
            for feat in Features :
                Etage= feat["etage_final"]
                if feat["nom_dept"] not in List_departs :
                    List_departs.append(feat["nom_dept"])
            
            # Sélection par localisation des entités administratives grâce aux formations géologiques déjà isolées
            Fonctions_QGIS.Selectbylocation(self,self.Couche_Depart,Formations_Geol,False,0)

            # Récupération des éléments sélectionnés dans la couche
            Departs=self.Couche_Depart.selectedFeatures()
            # Récupération des noms des entités administratives
            for Depart in Departs :
                Nom=Depart[self.Colonne_D]
                # print("Nom_dept = "+Nom) # debugtest
                # print(List_departs) # debugtest

                if Nom not in List_departs :
                    Message="Recherche de formations complémentaires en {}...".format(Nom)
                    Pourcent=50
                    Utilitaires.Progression(self.Bar_1,self.progres_label,Message,Pourcent)

                    # Chargement des flux WFS en plus pour vérification
                    Utilitaires.Prepare_WFS(self,Nom)
                    # Sélectionner la couche en fonction de son nom
                    WFS=QgsProject.instance().mapLayersByName(Nom)[0]

                    Buffer=Fonctions_QGIS.Buffer(self,Formations_Geol,10000,False,True)
                    # Sélection par localisation des formations géologiques du WFS dans le tampon de la ville
                    Fonctions_QGIS.Selectbylocation_simple(self,WFS,Buffer,0)

                    # Utilisation de la boucle de vérification pour sélectionner les formations complémentaires
                    self.Boucle_verif(Formations_Geol,WFS,"ETAGE_FINAL",Etage)

                    # Récupération des éléments sélectionnés dans la couche
                    Features=WFS.selectedFeatures()
                    if len(Features)!=0:
                        # Ajouter les entités sélectionnées dans la couche en cours de complétion 
                        Layer_data.addFeatures(Features)
                    QgsProject.instance().removeMapLayer(WFS)


    def Cas_2(self,Formations_Geol,Layer_data):
        """
        Entrees : Formations_Geol (VectorLayer): Couche intermédiaire qui contient les formations géologiques sélectionnées avant l'agrégation 
                  Layer_data (informations d'une VectorLayer): Accès aux données de la couche en cours de modification
        Fonction : Sélection de formations géologiques complémentaires à partr du tampon de la ville
        """
        Repeteur=True

        # Récupération des éléments sélectionnés dans la couche
        Features=self.GDR_SILEX.selectedFeatures()
        # print("AA nb select = "+str(len(Features))) # debugtest
        if len(Features)!=0:
            # Utilisation de la boucle de vérification pour sélectionner les formations complémentaires
            self.Boucle_verif(Formations_Geol,self.GDR_SILEX,self.champ,self.Strati)

        # Sélection tant que des formations complémentaires dans la zone sont sélectionnables
        while Repeteur  :
            # Récupération des éléments sélectionnés dans la couche
            Features=self.GDR_SILEX.selectedFeatures()
            # print("T nb select = "+str(len(Features))) # debugtest
            if len(Features)>1:
                # Ajouter les entités sélectionnées dans la couche en cours de complétion 
                Layer_data.addFeatures(Features)
                # Utilisation de la boucle de vérification pour sélectionner les formations complémentaires
                self.Boucle_verif(Formations_Geol,self.GDR_SILEX,self.champ,self.Strati)
            else :
                # Ajouter les entités sélectionnées dans la couche en cours de complétion 
                Features=Formations_Geol.getFeatures()
                # Récupération de l'étage final des entités sélectionnées pour l'utiliser ensuite pour compléter la sélection
                test= Formations_Geol.featureCount()
                if test >=0 :
                    for feat in Features :
                        Etage= feat["etage_final"]
                        # print(Etage) # debugtest
                else :
                    Etage=self.Liste_Strati.currentText()

                # Sélection par localisation des entités administratives grâce à l'étage final des formations géologiques déjà isolées
                Fonctions_QGIS.Selectbylocation_simple(self,self.Couche_Depart,Formations_Geol,0)
                Fonctions_QGIS.Selectbylocation_simple(self,self.Couche_Depart,self.Buffer,1)

                # Récupération des éléments sélectionnés dans la couche
                # print(str(self.Couche_Depart.name())) # debugtest
                Departs=self.Couche_Depart.selectedFeatures()
                # Récupération des noms des entités administratives

                Select=len(self.GDR_SILEX.selectedFeatures())
                Message="Recherche de polygones complémentaires (Nb Select : {})".format(Select)
                Pourcent=50
                Utilitaires.Progression(self.Bar_1,self.progres_label,Message,Pourcent)

                for Depart in Departs :
                    Nom=Depart[self.Colonne_D]
                    # print(Nom) # debugtest
                    
                    Message="Chargement du Flux WFS ({})...".format(str(Depart[self.Colonne_D]))
                    Pourcent=60
                    Utilitaires.Progression(self.Bar_1,self.progres_label,Message,Pourcent)

                    # Chargement des flux WFS en plus pour vérification
                    Utilitaires.Prepare_WFS(self,Nom)
                    # Sélectionner la couche en fonction de son nom
                    WFS=QgsProject.instance().mapLayersByName(Nom)[0]

                    Message="Recherche de formations complémentaires en {}...".format(str(Depart[self.Colonne_D]))
                    Pourcent=60
                    Utilitaires.Progression(self.Bar_1,self.progres_label,Message,Pourcent)

                    # Sélection par localisation des formations géologiques du WFS dans le tampon de la ville
                    Fonctions_QGIS.Selectbylocation_simple(self,WFS,self.Buffer,0)
                    # Selection par attribut des formations recherchées dans le tampon
                    Etage=self.Liste_Strati.currentText()
                    Fonctions_QGIS.Selectbyattribute(self,WFS,"ETAGE_FINAL",Etage,3)

                    # Récupération des éléments sélectionnés dans la couche
                    Features=WFS.selectedFeatures()
                    # print("nb select Ter = "+str(len(Features))) # debugtest
                    Bool=True
                    # Seconde boucle pour sélectionner les formations géologiques spécifiques au WFS
                    while Bool :
                        if len(Features)>1:
                            # Ajouter les entités sélectionnées dans la couche en cours de complétion 
                            Layer_data.addFeatures(Features)
                            self.Boucle_verif(Formations_Geol,WFS,'ETAGE_FINAL',Etage)
                            Features=WFS.selectedFeatures() # debugtest
                            # print("nb select Cash = "+str(len(Features))) # debugtest
                        else :
                            Bool=False
                    QgsProject.instance().removeMapLayer(WFS)
                
                Select=len(self.GDR_SILEX.selectedFeatures())
                Message="Ajout des polygones complémentaires (Nb Select : {})".format(Select)
                Pourcent=70
                Utilitaires.Progression(self.Bar_1,self.progres_label,Message,Pourcent)

                # Déselection des entités pour faciliter les recherches et éviter les doublons
                self.Couche_Depart.removeSelection()
                Repeteur=False

        Select=len(self.GDR_SILEX.selectedFeatures())
        Message="Nombre de formations sélectionnées (Nb Select : {})".format(Select)
        Pourcent=80
        Utilitaires.Progression(self.Bar_1,self.progres_label,Message,Pourcent)
        
                    
    def Couche_Finale(self):
        """
        Fonction : Création de la couche finale une fois l'agrégation des entités de chaque type de silicites réalisés
        """
        self.Bar_1.reset()
        Message="Regroupement des polygones dans la couche finale"
        Pourcent=50
        Utilitaires.Progression(self.Bar_1,self.progres_label,Message,Pourcent)

        # Récupération du nom de la nouvelle couche
        Nom=self.Nom_Couche.text()
        if Nom=="":
            Nom="Formations_Geol"
        # Sélectionner la couche en fonction de son nom
        Entree=QgsProject.instance().mapLayersByName(Nom)

        # Aggrégation des entités de la couche intermédiaire pour faciliter les sélections
        Fusion=Fonctions_QGIS.Agregation(self,Entree[0])
        # Ajout d'un champs supplémentaire pour conserver le nom du type de silicites/étage géologique
        Fusion.dataProvider().addAttributes([QgsField("Nom_extraction", QVariant.String)])
        Fusion.updateFields()

        # Récupération de l'index du champ supplémentaire
        field_idx = Fusion.fields().indexOf('Nom_extraction')
        with edit(Fusion):
            for feature in Fusion.getFeatures():
                # Changement de la valeur du champ
                Fusion.changeAttributeValue(feature.id(), field_idx, Nom)

        # Vérifier si une des couches n'est pas déjà chargée
        if QgsProject.instance().mapLayersByName("Couches_Geologiques"):
            # Sélectionner la couche en fonction de son nom
            Couche_finale=QgsProject.instance().mapLayersByName("Couches_Geologiques")[0]
        else:
            # Créer la couche si elle n'éxiste pas
            Couche_finale=Utilitaires.Creation_Couche(self,"Polygon","3857","Couches_Geologiques",Fusion)
            # Définition du style de la couche
            style_path = os.path.dirname(os.path.abspath(__file__))+'//Styles//Legende_CODE_HEXA_ETAGE_FINAL_vEV.qml'
            if os.path.exists(style_path):
                Couche_finale.loadNamedStyle(style_path)
            else:
                pass
        
            # Ajout de la couche à la carte
            QgsProject.instance().addMapLayer(Couche_finale)
        
        # Récupération des entités de la couche aggrégée
        Features=Fusion.getFeatures()
        attr = Couche_finale.dataProvider().fields().toList()
        # Ajouter les entités sélectionnées dans la couche finale
        Couche_finale.dataProvider().addFeatures(Features)
        
        Message="Avancement dans la récupération des formations géologiques"
        self.progres_label.setText(Message)
        self.Bar_1.reset()

        self.Choix_WFS.clear()
        self.Liste_Strati.clear()
        self.Nom_Couche.clear()

        self.Choix_Champ.setEnabled(False)
        self.Liste_Zone.setEnabled(False)
        self.OK_Zone_2.setEnabled(False)
        self.Choix_Admin.setEnabled(False)
        self.OK_Admin.setEnabled(False)
        self.Choix_Depart.setEnabled(False)
        self.OK_Zone_3.setEnabled(False)
        self.Launch.setEnabled(False)
        self.Colonne_Nom.setEnabled(False)
        self.Choix_WFS.setEnabled(False)
        self.OK_Depart.setEnabled(False)
        self.Bouton_Etage.setChecked(True)
        self.Liste_Strati.setEnabled(False)
        self.OK_Strati.setEnabled(False)
        self.Final_Launch.setEnabled(False)
